import sys

from PySide2.QtUiTools import QUiLoader
from PySide2.QtWidgets import QApplication, QPushButton, QLineEdit, QPlainTextEdit, QFileDialog, QTreeWidget, QTreeWidgetItem, QComboBox
from PySide2.QtCore import QFile, QObject

from cut import *

class MainWindow(QObject):
	def __init__(self, parent=None):
		super(MainWindow, self).__init__(parent)

		ui_file = QFile("AlbumGui.ui")
		ui_file.open(QFile.ReadOnly)
		loader = QUiLoader()
		self.window = loader.load(ui_file)
		ui_file.close()

		self.browseFile = self.window.findChild(QPushButton, "browseFile")
		self.movieFile = self.window.findChild(QLineEdit, "movieFile")
		self.inputTracks = self.window.findChild(QPlainTextEdit, "inputTracks")
		self.regex = self.window.findChild(QLineEdit, "regex")
		self.extractFunction = self.window.findChild(QComboBox, "extractFunction")
		self.albumName = self.window.findChild(QLineEdit, "albumName")
		self.albumPath = self.window.findChild(QLineEdit, "albumPath")
		self.run = self.window.findChild(QPushButton, "run")

		self.outputTracks = self.window.findChild(QTreeWidget, "outputTracks")
		self.parsingResult = self.window.findChild(QPlainTextEdit, "parsingResult")


		self.browseFile.clicked.connect(self.on_browse_file)
		self.inputTracks.textChanged.connect(self.on_reparse)
		self.regex.textChanged.connect(self.on_reparse)
		self.movieFile.textChanged.connect(self.on_movie_file_changed)
		self.extractFunction.currentIndexChanged.connect(self.on_reparse)
		self.run.clicked.connect(self.on_run)

		self.window.show()

	def on_browse_file(self):
		filename = QFileDialog.getOpenFileName(dir=r"C:\Users\Jonathan\Desktop\Album/")
		self.movieFile.setText(filename[0])

	def on_movie_file_changed(self):
		# extract album name from input file:
		fn = self.movieFile.text()
		albumName = fn[fn.rfind('/')+1 : fn.rfind('.')]
		albumPath = fn[ : fn.rfind('/')+1]
		self.albumName.setText(albumName)
		self.albumPath.setText(albumPath)

	def extract(self, tracks, regex):
		if self.extractFunction.currentText() == "Time - Name":
			return ExtractTimeName(tracks, regex)
		elif self.extractFunction.currentText() == "Name - Time":
			return ExtractNameTime(tracks, regex)
		else:
			raise Exception("unknown extraction method")

	def on_reparse(self):
		tracks = self.inputTracks.toPlainText()
		regex = self.regex.text()
		self.outputTracks.clear()
		self.parsingResult.clear()

		try:
			timeStamps, names = self.extract(tracks, regex)
			for i, (time, name) in enumerate(zip(timeStamps, names)):
				c = [0 if not s else int(s) for s in time[1:]]
				time_str = "{:02d}:{:02d}:{:02d}".format(*c)
				file_name = slugify(name)
				self.outputTracks.insertTopLevelItem(i, QTreeWidgetItem([str(i+1), time_str, name, file_name]))
			for i in range(4): #resize columns
				self.outputTracks.resizeColumnToContents(i)
		except Exception as e:
			self.parsingResult.clear()
			self.parsingResult.appendPlainText(str(e))


	def on_run(self):
		tracks = self.inputTracks.toPlainText()
		regex = self.regex.text()
		timeStamps, trackNames = self.extract(tracks, regex)
		fileNames = [slugify(name) for name in trackNames]
		path = self.albumPath.text()
		movieFile = self.movieFile.text()
		albumName = self.albumName.text()

		Cut(path, movieFile, timeStamps, fileNames, trackNames, albumName)


if __name__ == '__main__':
	app = QApplication(sys.argv)
	form = MainWindow()
	sys.exit(app.exec_())