
import re
import subprocess
import slugify as python_slugify #https://github.com/un33k/python-slugify
import os.path
import os

def slugify(string): #  pip install python-slugify
	return python_slugify.slugify(string, lowercase=False, separator=" ")


def Seconds(timeStamp):
	# timeStamp[0] will contain the outer group matched by the regex - ugly but hmg.
	if not timeStamp[1]:
		HH = 0
	else:
		HH = int(timeStamp[1])
		
	if not timeStamp[2]:
		MM = 0
	else:
		MM = int(timeStamp[2])
		
	if not timeStamp[3]:
		SS = 0
	else:
		SS = int(timeStamp[3])
		
	return HH*3600 + MM*60 + SS


# takes a list of timestamps and names and cuts the input into pieces, naming them accordingly
def Cut(path, fAudio, timeStamps, fileNames, trackNames, albumName, stopAtLast=False):
	if not os.path.isfile(fAudio):
		print("\nERROR: Input file does not exist: ", fAudio)
		return
		
	assert(len(trackNames) == len(fileNames))
	assert(len(trackNames) == len(timeStamps))
	
	try:
		os.mkdir(path+albumName)
	except FileExistsError:
		print('Output directory already exists. Continuing...')
	path += albumName+'\\'
	
	for i in range(len(timeStamps)-1):
		start_time = Seconds(timeStamps[i])
		end_time = Seconds(timeStamps[i+1])
		output = path+str(i+1).zfill(2)+" "+fileNames[i]+".mp3"
		if(os.path.isfile(output)):
			print("\nERROR: File already exists: ", output)
			return
		
		print(i+1, "/", len(timeStamps))
		cmd = 'ffmpeg -ss {start} -t {duration} -i "{input_file}" -metadata album="{album}" -metadata title="{title}" -metadata track={track} "{output_file}"'.format(
			start=start_time,
			duration=end_time - start_time,
			input_file=fAudio,
			album=albumName.replace('"', r'\"'),
			title=trackNames[i].replace('"', r'\"'),
			track=i+1,
			output_file=output)
		subprocess.run(cmd, shell=True, check=True)
	
	# final segment until the end of the Album
	if not stopAtLast:
		output = path+str(len(fileNames)).zfill(2)+" "+fileNames[-1]+".mp3"
		print(len(timeStamps), "/", len(timeStamps))
		cmd = 'ffmpeg -ss {start} -i "{input_file}" -metadata album="{album}" -metadata title="{title}" -metadata track={track} "{output_file}"'.format(
			start=Seconds(timeStamps[-1]),
			input_file=fAudio,
			album=albumName.replace('"', r'\"'),
			title=trackNames[-1].replace('"', r'\"'),
			track=len(timeStamps),
			output_file=output)
		subprocess.run(cmd, shell=True)
	
	print("\ndone")
	

# Various functions to extract timestamps and names from lists
def ExtractNameTime(content, regex):
	rTimes = re.compile(regex)
	timeStamps = rTimes.findall(content)
	names = list()
	P = [(m.start(0), m.end(0)) for m in rTimes.finditer(content)]
	names.append(content[0:P[0][0]].strip())
	for i in range(len(P)-1):
		names.append(content[P[i][1]:P[i+1][0]].strip())

	return (timeStamps, names)


def ExtractTimeName(content, regex):
	rTimes = re.compile(regex)
	timeStamps = rTimes.findall(content)
	names = list()
	P = [(m.start(0), m.end(0)) for m in rTimes.finditer(content)]
	for i in range(len(P)-1):
		names.append(content[P[i][1]:P[i+1][0]].strip())

	names.append(content[P[-1][1]:].strip())
	return (timeStamps, names)



# song names are converted to valid filenames using slugification:
def Do(path, fAudio, fChapters, Extract, regex = "((\d?\d):)?(\d?\d):(\d?\d)", showOnly=False, fixNames=True, stopAtLast=False):
	albumName = fAudio[:fAudio.rfind('.')]
	print(albumName)
	
	fAudio = path+fAudio
	fChapters = path+fChapters
	with open(fChapters, "rt") as f:
		content = f.read()
	timeStamps, fileNames = Extract(content, regex)
	trackNames = fileNames
	if fixNames:
		fileNames = [slugify(name) for name in fileNames]
	for i in range(len(timeStamps)):
		print(timeStamps[i], " - ", fileNames[i])
	if not showOnly:
		Cut(path, fAudio, timeStamps, fileNames, trackNames, albumName, stopAtLast)
